#!/usr/bin/perl

# patches

# A perl script adapated from JW shell scripts to use basic potential FDPB
# calculations from a variety of 'fiddled' charge sets, and then contour into
# 2D patches of potential on the shrink-wrap grid for subsequent analysis.

# Adapted March 2006 from sites_identify.pl, Jim Warwicker, UoM

# Five basic charge set manipulations to look at different properties:

#	standard		takes gromos charge set with std pks (at given pH)
#	unit			uniform charge density on protein
#	pkshift			takes pk calculation results to adjust charges
#	pkburied		charge only where have buried ionisable gp and 3<pred pk<10
#	profile			charges are (0-1) seq profile weights

# For each of these modes, plus depending on what FDPB parameter (.ctr) file we use, we
# must make sure that we apply the appropriate E/nonE threshold value

# control the run type (above) by setting the following parameters;

# standard
#	use normal (probably GROMOS-derived) charge set
#	don't run escalc_pkupdate
# unit
#	use uniform charge density of protein
#	don't run escalc_pkupdate
# pkshift
#	use charge set at $pkpath
#	run escalc_pkupdate / SEQ_FIDDLE = 'no', NODH_FIDDLE = 'no', set ESCALC_FILE and pkCALC_FILE
# pkburied
#	use charge set at $pkpath
#	run escalc_pkupdate / SEQ_FIDDLE = 'no', NODH_FIDDLE = 'yes', set ESCALC_FILE and pkCALC_FILE
# profile
#	use profile at $profile_path
#	run escalc_pkupdate / SEQ_FIDDLE = 'yes', NODH_FIDDLE = 'no', set SEQ_FILE and COORD_FILE

# pdbs/chains for calculation are assumed to be in file 'pdbs.list', one per record (e.g. 1abc or 1abcA)
# if onechain = 'yes', we assume a 5 char id, with last char = chain; 4 char id otherwise

# For the patches version of this program, a number of restraints are currently active (apr 06):

#	we are set up purely for +/-/np patch contours at present - so use 'standard' mode only
#	this (standard mode) is subject to change - e.g. we could patch-up clustered profile o/p

#	in the 'standard' mode we get too fragmented patches if we use polar/dipolar groups,
#	so use the escalc_full set-up - just like a normal GRASP run (pH still flexible)

#	grid step is changeable currently by selecting different fdcalc ctr files

# set PATHS for binaries, charge data files, pdbs

$ENV{BIN}		= './';
$ENV{EXAMPLES}		= './';
$ENV{PDB_PATH}		= './';

# ----------- BELOW are the parameters we will change most frequently

$mode			= 'standard';		# standard, unit, pkshift, pkburied, profile
if ($mode ne 'standard') {
	print "must have standard mode currently\n";
	exit;
}
$onechain		= 'yes';		# yes/no to extract/not one chain from each pdb
$remove_maps		= 'yes';		# mostly we dont want to look at maps (they are big!)

$profile_path		= '??/';		# profile: where we have profiles stored (if any)
$prof_ext		= '.profile';		# profile: profile file = $pdb.$prof_ext
$pkpath			= '??/';		# pkshift/pkburied: where we have pk and escalc files (pkshift, pkburied)
$es_ext			= '.qop';		# pkshift/pkburied: escalc file = $pdb.$es_ext
$pk_ext			= '_VdW12.pk';		# pkshift/pkburied: pk file = $pdb.$pk_ext

$ENV{PEAK_DIST}		= 'no';			# yes/no to calculate/not distance to site (need known site aas)
$ENV{CYS_INCLUDE}	= 'no';			# yes/no to put ionisable cys into calculation/not (make_escalc)
$ENV{TYR_IONISABLE}	= 'no';			# yes/no to include TYR (ionisable) or TYRH/not (pdb_es)

$ENV{PATCH_CONTOUR}	= '50';			# mv value (+ve) at which to contour -/+
$ENV{NP_CONTOUR}	= '20';			# mV value (+ve) between - and + levels to contour np
						# NP_CONTOUR must be .le. PATCH_CONTOUR (no overlap)

$ENV{PH}		= '5.3';		# pH and intrinsic pks used to get charges
#system ("cp $ENV{EXAMPLES}/fdcalc_grid20_pH63_patch.ctr fdcalc.ctr");	# copy grid parameter file
system ("cp $ENV{EXAMPLES}/fdcalc_grid20_pH53_patch_allQ.ctr fdcalc.ctr");	# copy grid parameter file

# ----------- ABOVE are the parameters we will change most frequently

if ($mode eq 'profile') {
	$ENV{SEQ_FIDDLE}	= 'yes';
} else {
	$ENV{SEQ_FIDDLE}	= 'no';
}
if ($mode eq 'pkburied') {
	$ENV{NODH_FIDDLE}	= 'yes';	# use the NODH and pk range condition
} else {
	$ENV{NODH_FIDDLE}	= 'no';		# just use predicted pks (if $mode = 'pkshift')
}

if ($mode eq 'unit') {				# look for a uniform charge density file ('unit')
	$ENV{DATA}		= '/fs/storage/home/mjfssjw2/progs/data/efld_tmp3/';
} else {
	$ENV{DATA}		= '/fs/storage/home/mjfssjw2/progs/data/';
}

# set various other parameters for the calculation

#$ENV{PH}		= '7.0';		# pH and intrinsic pks used to get charges for ??
$ENV{WHICH_VDW}		= '1';			# set of VdW radii (leave at 1)
$ENV{which_vdw}		= '1';			# ditto
$ENV{WATERS}		= 'F';			# T/F = do/do not include water molecules (pdb_es)
$ENV{OTHER_HETATM}	= 'F';			# T/F = do/do not include hetero groups (pdb_es)
$ENV{PROTEIN}		= 'T';			# T/F = do/do not include protein (pdb_es)
$ENV{NUCLEIC}		= 'F';			# T/F = do/do not include nucleic acid (pdb_es)

$ENV{MOLECULE_COORDS}	= 'tmp';		# leave this (just swapping files around)

open (PDBS, "pdbs.list") or die "cannot open pdbs.list\n";
@pdblist=<PDBS>;
close (PDBS);

foreach $pdb (@pdblist)	{
	chomp $pdb;
	if ($onechain eq 'yes') {
		$pdbfile	= substr($pdb,0,4);	# chars 1-4 are pdb id
		$chain		= substr($pdb,4,1);	# char 5 is chain id
	} else {
		$pdbfile	= $pdb;
	}
	print "processing $pdb\n";

	system ("cp $ENV{PDB_PATH}$pdbfile'.pdb' $ENV{MOLECULE_COORDS}'.pdb'");

	$ENV{ONE_MODEL_IN}	= $ENV{MOLECULE_COORDS}; # Jun 06: one model (NMR files) before extract_chain
	system ("$ENV{BIN}/one_model");	       		# get all up to ENDMDL (everything if no ENDMDL)
	system ("rm -f $ENV{MOLECULE_COORDS}'.pdb'");
	system ("mv one_model_out.pdb $ENV{MOLECULE_COORDS}'.pdb'");

	if ($onechain eq 'yes') {
		system ("mv $ENV{MOLECULE_COORDS}'.pdb' extract_in.pdb");
		$ENV{CHAIN_GET}	= $chain;
		system ("$ENV{BIN}/extract_chain");
		system ("mv extract_out.pdb $ENV{MOLECULE_COORDS}'.pdb'");
		system ("rm -f extract_in.pdb");
	}
	if ($ENV{PEAK_DIST} eq 'yes') {		# get site gps if we want to check prediction geometry/distance
		system ("$ENV{BIN}/make_as_gps");
	}
	system ("$ENV{BIN}/pdb_es");			# 'clean' the coordinates
	system ("rm -f pdb_es_temp.pdb");	# lnx add on since can't 'DISPOSE'
	system ("mv pdb_es.inf 'pdb_es_'$ENV{MOLECULE_COORDS}'.inf'");

	if ($mode eq 'standard' or $mode eq 'unit') {	# get the charges (through escalc)

		system ("$ENV{BIN}/make_polh_gen_ctr");	# generate polar hydrogens
		system ("cp 'polh_gen_'$ENV{MOLECULE_COORDS}'.ctr' polh_gen.ctr");
		system ("$ENV{BIN}/polh_gen");
		system ("rm -f polh_gen.ctr");
		system ("mv polh_gen.inf 'polh_gen_'$ENV{MOLECULE_COORDS}'.inf'");
		$ENV{ALL_CHARGES}	= 'YES';	# (all charges)
		system ("$ENV{BIN}/make_escalc_ctr");
#		system ("cp 'es_'$ENV{MOLECULE_COORDS}'_full.ctr' escalc.ctr");
		system ("cp 'es_'$ENV{MOLECULE_COORDS}'_all.ctr' escalc.ctr");
		system ("$ENV{BIN}/escalc");
		system ("rm -f escalc.ctr");

	} else {					# use escalc_pkupdate

		if ($mode eq 'profile') {
			$ENV{SEQ_FILE}		= $profile_path.$pdb.$prof_ext;	# profile
			$ENV{COORD_FILE}	= $pdb.'.pdb'			# coordinates ??
		} else {				# $mode = pkshift or pkburied
			$ENV{ESCALC_FILE}	= $pkpath.$pdb.$es_ext;		# escalc file for pk shifting
			$ENV{pkCALC_FILE}	= $pkpath.$pdb.$pk_ext;		# pk_calc fiel for pk shifting
		}
		system ("$ENV{BIN}/escalc_pkupdate");		# fiddle charge update wirh pks or profile
		system ("mv upd_es_tmp_all.qop es_tmp_all.qop");
		system ("mv escalc_pkupdate.inf $pdb'_pkupdate.inf'");

	}

#	system ("cp $ENV{EXAMPLES}/fdcalc_grid20_pH63_patch.ctr fdcalc.ctr");	# copy grid parameter file
	system ("$ENV{BIN}/fdcalc");			# FDPB potential map calculation
#	system ("rm -f fdcalc.ctr");

#	system ("mv fd_tmp.pot 'fd_'$pdbfile'.pot'");
#	system ("mv fd_tmp.mol 'fd_'$pdbfile'.mol'");
#	system ("mv fd_tmp.itr 'fd_'$pdbfile'.itr'");
	system ("mv fd_tmp.pot 'fd_'$pdb'.pot'");
	system ("mv fd_tmp.mol 'fd_'$pdb'.mol'");
	system ("mv fd_tmp.itr 'fd_'$pdb'.itr'");
	system ("rm -f fd_tmp.*");

#	$ENV{MOLECULE}		= $pdbfile;
	$ENV{MOLECULE}		= $pdb;			# June 2006 update to carry chain id to peaks o/p
	system ("$ENV{BIN}/patch_contour");		# process FDPB map to contour and get patches
#	system ("rm -f $pdb'.pdb'");

# October 2007: add ASA calculation.
	system ("cp tmp_es.pdb sa_calc.in");
	system ("$ENV{BIN}/sa_calc");
	system ("mv sa_calc.out $pdb'.sa'");
	system ("rm -f sa_calc.in");

	system ("rm -f es_tmp*.*");		# cleanup for each pdb
	system ("rm -f polh*tmp*.*");
	system ("rm -f polh_gen.pep");
	system ("rm -f tmp*.pdb");
	system ("rm -f pdb_es_tmp.*");
#	system ("rm -f 'fd_'$pdbfile'.pot'");
#	system ("rm -f 'fd_'$pdbfile'.mol'");
#	system ("rm -f 'fd_'$pdbfile'.itr'");
	system ("rm -f 'fd_'$pdb'.pot'");
	system ("rm -f 'fd_'$pdb'.mol'");
	system ("rm -f 'fd_'$pdb'.itr'");
	if ($remove_maps eq 'yes') {
		system ("rm -f *.txt");
		system ("rm -f *map.pdb");
	}

	system ("rm -f *_patch.pdb");
	system ("rm -f *_peaks.inf");
	system ("rm -f *_PATCHES.pdb");
	system ("rm -f *_SHELL.pdb");

}

system ("rm -f fdcalc.ctr");

exit;
